set more off
set linesize 240
clear all
capture log close
log using ch14_large.txt,text replace
*************************************************************
* ch14_1.do                    
* replication files for Stock-Watson 4th Editions Chapter 14
************************************************************
 
* Read In Data 
* (Note: Change path name so that it is appropriate for your computer);
use "/Users/mwatson/Dropbox/TB/4E/WebSupplements/ForStudents/SW_4E_ReplicationFiles/SW_4E_Replication_Data/ca_school_testscore_insample.dta"
append using "/Users/mwatson/Dropbox/TB/4E/WebSupplements/ForStudents/SW_4E_ReplicationFiles/SW_4E_Replication_Data/ca_school_testscore_outofsample.dta"

generate insample = _n <= 1966
generate outofsample = _n > 1966
desc
**** Small Data Set Prediction
# delimit ;
global pred_1 "
str_s 
med_income_z
te_avgyr_s
exp_1000_1999_d
frpm_frac_s
ell_frac_s
freem_frac_s
enrollment_s
fep_frac_s
edi_s
re_aian_frac_s
re_asian_frac_s
re_baa_frac_s
re_fil_frac_s
re_hl_frac_s
re_hpi_frac_s
re_tom_frac_s
re_nr_frac_s
te_fte_s
te_1yr_frac_s
te_2yr_frac_s
te_tot_fte_rat_s
exp_2000_2999_d
exp_3000_3999_d
exp_4000_4999_d
exp_5000_5999_d
exp_6000_6999_d
exp_7000_7999_d
exp_8000_8999_d
expoc_1000_1999_d
expoc_2000_2999_d
expoc_3000_3999_d
expoc_4000_4999_d
expoc_5000_5999_d
revoc_8010_8099_d
revoc_8100_8299_d
revoc_8300_8599_d
revoc_8600_8799_d
";
# delimit cr

local i = 0
* Name Levels construct raw data and standardized data
foreach x in $pred_1{
	local i = `i'+1
	generate x_`i'= `x'
	quiet sum x_`i' if insample
	generate s_x_`i' = (x_`i'-r(mean))/r(sd)
}

forvalues i = 1/38 {
	forvalues j = `i'/38 {
	generate xx_`i'_`j'=x_`i'*x_`j'
	quiet sum xx_`i'_`j' if insample
	generate s_xx_`i'_`j' = (xx_`i'_`j'-r(mean))/r(sd)
  }
}

forvalues i = 1/38 {
	generate xxx_`i'=(x_`i')^3
	quiet sum xxx_`i' if insample
	generate s_xxx_`i' = (xxx_`i'-r(mean))/r(sd)
}

quiet summarize testscore if insample
generate testscore_dm = testscore-r(mean)

* Drop a perferectly collinear regressor
drop xx_1_19
drop s_xx_1_19

global pred_x "x_* xx_* xxx_*" 
global pred_sx = "s_x_* s_xx_* s_xxx_*" 


* Compute Cross-fold RMSE using regression in-sample* Only doing OLS prediction, so no need to used standardized values
quiet regress testscore $pred_x if insample
predict testscore_ols
generate e_ols = testscore-testscore_ols
generate e_ols2 = e_ols^2
quiet sum e_ols2 if outofsample
scalar rmse_oos_ols = sqrt(r(mean))
***** Compute in-sample RMSE by Cross-validation
** This uses the ADO file crossfold.ado .. it computes RMSE for each of k-folds .. stored in the vector r(est). I compute overall RMSE by averaging RMSEs and taking square root
quiet crossfold regress testscore $pred_x if insample, k(10) 
matrix a = r(est)'*r(est)
scalar rmse_cv_is_ols = sqrt(a[1,1]/10)
*** Print Results
display "OLS RMSE In-Sample by 10-fold CV = " rmse_cv_is_ols
display "OLS RMSE Out-of-Sample = " rmse_oos_ols
*
* Compute Regression using standardized regressors (Results shown in Table 14.4)
regress testscore_dm $pred_sx if insample, noconstant

*
***********************************************************************************
* PCA -- NOTE .. This code does not do cross-validation to choose the number of PCs
***********************************************************************************

mkmat $pred_sx, mat(xvarsmat)
*
local npc = 46
* Estimate on the full insample
quiet pca $pred_sx if insample, components(`npc')
* create the principal components
matrix ev = e(Ev)
matrix ev = ev'
scalar trace_var = e(trace)
matrix ev_norm = (1/trace_var)*ev
display "Scree Plot Values for Figure 14.6"
matrix list ev_norm
matrix evec = e(L)
matrix pc = xvarsmat*evec
svmat pc
quiet regress testscore_dm pc1-pc`npc' if insample, noconstant
predict testscore_dm_pc 

**** PC coefficients (Results shown in Table 14.4)
matrix b_pc = e(b)'
matrix b_pca_std = evec*b_pc
display "Coefficients on standardized Regressors from PCA Regression:"
matrix list b_pca_std

generate e_pc = testscore_dm-testscore_dm_pc
generate e_pc2 = e_pc^2
quiet sum e_pc2 if outofsample
scalar rmse_oos_pca = sqrt(r(mean))
display "PCA RMSE Out-of-Sample = " rmse_oos_pca

*********************************************************
*** Ridge .. this is elasticnet with alpha = 0
*********************************************************
elasticnet linear testscore $pred_x if insample, alpha(0) log ///
   fold(10) grid(100, min(.0001)) alllambdas
   cvplot
cvplot
graph export "cvplot_large_ridge.png", replace
scalar lambda_ridge = e(lambda_sel)
scalar rmse_cv_is_ridge = sqrt(e(cvm_min))
estimates store ridge

**** Ridge coefficients (Results shown in Table 14.4)
display "Coefficients on standardized Regressors from Ridge Regression:"
lassocoef, display(coef)

predict testscore_ridge 
generate e_ridge = testscore-testscore_ridge
generate e_ridge2 = e_ridge^2
quiet sum e_ridge2 if outofsample
scalar rmse_oos_ridge = sqrt(r(mean))
display "Ridge RMSE In-Sample by 10-fold CV = " rmse_cv_is_ridge
display "Estimate of lamda (Stata normalization) = " lambda_ridge
** Note SW Ch 14 uses a different normalization for lambda, which is Stata's times sample size
scalar lambda_ridge_sw = lambda_ridge*1966
display "Estimate of lamda (SW 4e) = " lambda_ridge_sw
display "Ridge RMSE Out-of-Sample = " rmse_oos_ridge


*****************************************************************************
*** Lasso .. this is elasticnet with alpha = 1
***** Note Lasso takes considerable time to run (several hours on my machine)
***************************************************************************** 
elasticnet linear testscore $pred_x if insample, alpha(1) log ///
   folds(5) grid(50, min(.05)) alllambdas
cvplot
graph export "cvplot_large_lasso.png", replace
scalar lambda_lasso = e(lambda_sel)
scalar rmse_cv_is_lasso = sqrt(e(cvm_min))
estimates store lasso

**** Lasso coefficients (Results shown in Table 14.4)
display "Coefficients on standardized Regressors from Lasso Regression:"
lassocoef, display(coef)

predict testscore_lasso 
generate e_lasso = testscore-testscore_lasso
generate e_lasso2 = e_lasso^2
quiet sum e_lasso2 if outofsample
scalar rmse_oos_lasso = sqrt(r(mean))
display "Lasso RMSE In-Sample by 10-fold CV = " rmse_cv_is_lasso

display "Estimate of lamda (Stata normalization) = " lambda_lasso
** Note SW Ch 14 uses a different normalization for lambda, which is Stata's times 2*sample size
scalar lambda_lasso_sw = lambda_lasso*1966*2
display "Estimate of lamda (SW 4e) = " lambda_lasso_sw

display "Lasso RMSE Out-of-Sample = " rmse_oos_lasso


log close